﻿using System;
using System.Collections.Generic;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Problems.MultiObjectiveTestFunctions.Comparators;

namespace HeuristicLab.Problems.MultiObjectiveTestFunctions {
  /// <summary>
  /// The inverted generational Distance is defined as the mean of a all d[i]^(1/p),
  ///  where d[i] is the minimal distance the ith point of the optimal pareto front has to any point in the evaluated front.   
  /// </summary>
  public class InvertedGenerationalDistance : IMultiObjectiveDistance {
    private double p;

    public InvertedGenerationalDistance(double p) {
      if (p <= 0) throw new ArgumentOutOfRangeException("weighting factor p has to be greater than 0");
      this.p = 1 / p;
    }

    /// <summary>
    /// 
    /// </summary>
    /// <param name="front"></param>
    /// <param name="optimalFront"></param>
    /// <param name="p"></param>
    /// <returns></returns>
    public static double GetDistance(IEnumerable<double[]> front, IEnumerable<double[]> optimalFront, double p) {
      return new InvertedGenerationalDistance(p).Compare(front, optimalFront);
    }

    public double Compare(IEnumerable<double[]> front, IEnumerable<double[]> optimalFront) {
      return new GenerationalDistance(p).Compare(optimalFront, front);
    }
  }
}
