﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace HeuristicLab.Problems.MultiObjectiveTestFunctions.Comparators {
  class Utilities {
        /// <summary>
        /// 
        /// </summary>
        /// <param name="point"></param>
        /// <param name="list"></param>
        /// <param name="reflexive"></param>
        /// <returns>return -1 if list contains only rejected points</returns>
    public static double minDistance(double[] point, IEnumerable<double[]> list, bool reflexive) {
      double min = double.MaxValue;
      bool empty = true;
      foreach (double[] r in list) {
        empty = false;
        if (!reflexive && ReferenceEquals(r,point)) continue;
        double d = 0;
        for (int i = 0; i < r.Length; i++) {
          d += (point[i] - r[i]) * (point[i] - r[i]);
        }
        min = Math.Min(d, min);
      }
      if (empty) throw new ArgumentException("Fronts must not be empty");
      if (min == double.MaxValue) { return -1; }
      return Math.Sqrt(min);
    }

     public static IComparer<double[]>  getDimensionComparer(int dim, bool descending) {
      return new DimensionComparer(dim, descending);
    }


    private class DimensionComparer : IComparer<double[]> {
      private int dim;
      private int descending;

      public DimensionComparer(int dimension, bool descending) {
        this.dim = dimension;
        this.descending = descending ? -1 : 1;
      }

      #region IComparer<DoubleArray> Members

      public int Compare(double[] x, double[] y) {
        if (x[dim] < y[dim]) return -descending;
        else if (x[dim] > y[dim]) return descending;
        else return 0;
      }

      #endregion
    }


    public static int binarySearch(double x, double[][] list, int dim) {
      int low = 0;
      int high = list.Length - 1;
      while (high >= low) {
        int middle = (low + high) / 2;
        if (list[middle][dim] == x) return middle;
        if (list[middle][dim] < x) low = middle + 1;
        if (list[middle][dim] > x) high = middle - 1;
      }
      return -1; 
    }
  }





}
