﻿using System;
using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MultiObjectiveTestFunctions {
  [Item("SchafferN1", "Schaffer function N.1 for mulitobjective optimization from // https://en.wikipedia.org/wiki/Test_functions_for_optimization [30.11.2015]")]
  [StorableClass]
  public class SchafferN1 : MultiObjectiveTestFunction {

    public override double[,] Bounds(int objectives) {
      return new double[,] { { -1e5, 1e5 } };
    }

    public override bool[] Maximization(int objecitves) {
      return new bool[2];
    }

    public override int MinimumSolutionLength {
      get { return 1; }
    }
    public override int MaximumSolutionLength {
      get { return 1; }
    }


    public override int MinimumObjectives {
      get { return 2; }
    }

    public override int MaximumObjectives {
      get { return 2; }
    }


    public override double[] ReferencePoint(int objecitves) {
        return new double[] { 1e5, 1e5 };
    }


    public override IEnumerable<double[]> OptimalParetoFront(int objecitves) {
        return PFStore.get("Schaffer");
    }
    public override double BestKnownHypervolume(int objecitves) {
        return new Hypervolume(ReferencePoint(objecitves), Maximization(2)).GetHypervolume(OptimalParetoFront(objecitves));

    }

    [StorableConstructor]
    protected SchafferN1(bool deserializing) : base(deserializing) { }
    protected SchafferN1(SchafferN1 original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new SchafferN1(this, cloner);
    }

    public SchafferN1() : base() { }





    public override double[] Evaluate(RealVector r, int objectives) {
      if (objectives != 2) throw new ArgumentException("The Schaffer N1 problem must always have 2 objectives");
      if (r.Length != 1) return null;
      double x = r[0];

      //objective1
      double f0 = x * x;

      //objective0
      double f1 = x - 2;
      f1 *= f1;

      return new double[] { f0, f1 };
    }
  }
}
